use SOAP::Lite;
require "./client/src/Validators.pl";

# some definitions used in the calls
use constant {
    ENDPOINT => 'https://soatest.paysafecard.com/psc/services/PscService',
    USERNAME => 'USER_CLASSIC',
    PASSWORD => 'PASS',
    CURRENCY => 'EUR',
    AMOUNT => 0.01,
    OKURL => 'http://okurl',
    NOKURL => 'http://nokurl',
    PIN => '5000000000002517',
    MID => '1000001243',
    SERIALNUMBER => '5000000000002517;0.01',
    DISPOSITIONSTATE => 'S',
};

sub setup;

# Calls the CreateDisposition web service method.
#   
#   @param username
#            Username to use for the web service call.
#   @param password
#            Password to use for the web service call.
#   @param mtid
#            Merchant transaction id to use for the web service call. It must
#            be unique for each invocation.
#   @param subId
#            SubId to use for the web service call. The parameter is optional
#            and can be <code>null</code>.
#   @param amount
#            Amount to use for the web service call.
#   @param currency
#            Currency to use for the web service call. It must contain a 3
#            letter ISO 4217 currency code.
#   @param okUrl
#            OKUrl to use for the web service call.
#   @param nokUrl
#            NOKurl to use for the web service call.
#   @param merchantClientId
#            ID of the client to use for the web service call. This is
#            merchant's internal id to identify the client. The parameter is
#            optional and can be <code>null</code>.
#   @param pnUrl
#            Payment notification URL to use for the web service call. The 
#            parameter is optional and can be <code>null</code>.
#   @param clientIp
#            IP address of the client to use for the web service call. This is
#            the IP of the client when connecting to the merchant. The
#            parameter is optional and can be <code>null</code>.
#   @return Response of the CreateDisposition web service call. The call was
#           successful if resultCode and errorCode of the response are 0. For
#           list of possible error codes refer to merchant documentation.
#   
sub createDisposition {
    my($username, $password, $mtid, $subId, $amount, $currency, $okUrl, $nokUrl, $merchantclientid, $pnUrl, $clientIp) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateStringNotNull($okUrl, 'okUrl');
    validateStringNotNull($nokUrl, 'nokUrl');
    validateCurrency($currency, 'currency');

    return $client->createDisposition(
                      SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
                      SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                      SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
                      SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
	              SOAP::Data->name('{urn:pscservice}amount' => $amount)->type('float'),
		      SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'),
		      SOAP::Data->name('{urn:pscservice}okUrl' => $okUrl)->type('string'),
		      SOAP::Data->name('{urn:pscservice}nokUrl' => $nokUrl)->type('string'),
		      SOAP::Data->name('{urn:pscservice}merchantclientid' => $merchantclientid)->type('string'),
                      SOAP::Data->name('{urn:pscservice}pnUrl' => $pnUrl)->type('string'),
                      SOAP::Data->name('{urn:pscservice}clientIp' => $clientIp)->type('string'))
                      ->result;
}

#    Calls the AssignCardToDisposition web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param mtid
#             Merchant transaction id to use for the web service call. It must
#             be unique for each invocation.
#    @param subId
#             SubId to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param amount
#             Amount to use for the web service call.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @param pin
#             PIN number to use for the web service call.
#    @return Response of the AssignCardToDisposition web service call. The call
#            was successful if resultCode and errorCode of the response are 0.
#            For list of possible error codes refer to merchant documentation.
sub assignCardToDisposition {
    my($username, $password, $mtid, $subId, $amount, $currency, $pin) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateStringNotNull($pin, 'pin');
    validateCurrency($currency, 'currency');

    return $client->assignCardToDisposition(
                       SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
                       SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                       SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
                       SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
                       SOAP::Data->name('{urn:pscservice}amount' => $amount)->type('float'),
                       SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'),
                       SOAP::Data->name('{urn:pscservice}pin' => $pin)->type('string'))
                       ->result;
}

#    Calls the AssignCardsToDisposition web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param mtid
#             Merchant transaction id to use for the web service call. It must
#             be unique for each invocation.
#    @param subId
#             SubId to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param amount
#             Amount to use for the web service call.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @param cards
#             Card information (pin, password) to use for the web service call.
#             At least one (1) card and at most ten (10) cards can be present in
#             the request. Card passwords are optional and should be
#             <code>null</code> if no password is specified for the given card.
#    @param locale
#             Locale to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param acceptingTerms
#             Integer value indicating if the user accepts Paysafecard's terms
#             of use. Value should be 1 if user accepts terms of use.
#    @return Response of the AssignCardToDisposition web service call. The call
#            was successful if resultCode and errorCode of the response are 0.
#            For list of possible error codes refer to merchant documentation.
sub assignCardsToDisposition {
    my($username, $password, $mtid, $subId, $amount, $currency, $locale, $acceptingTerms, @cards) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateCurrency($currency, 'currency');
    validateCards(@cards);
    
    @params = (SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
               SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
               SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
               SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
               SOAP::Data->name('{urn:pscservice}amount' => $amount)->type('float'),
               SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'),
               SOAP::Data->name('{urn:pscservice}locale' => $locale)->type('string'),
               SOAP::Data->name('{urn:pscservice}acceptingTerms' => $acceptingTerms)->type('string'));
               
    foreach (@cards) {
        %$card_href = %$_;
        $pin = $card_href->{pin};
        $password = $card_href->{password};
        
        $data = SOAP::Data->name("{urn:pscservice}cards");
        
        if ($password) {
            $data->value(\SOAP::Data->value(SOAP::Data->name("{urn:pscservice}pin" => $pin)->type('string'),
                                            SOAP::Data->name("{urn:pscservice}password" => $password)->type('string')));
        } else {
            $data->value(\SOAP::Data->name("{urn:pscservice}pin" => $pin)->type('string'));
        }
        push(@params, $data);
    }
    return $client->assignCardsToDisposition(@params)->result;
}

#    Calls the ModifyDispositionValue web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param mtid
#             Merchant transaction id to use for the web service call. It must
#             be unique for each invocation.
#    @param subId
#             SubId to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param amount
#             Amount to use for the web service call.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @return Response of the ModifyDispositionValue web service call. The call
#            was successful if resultCode and errorCode of the response are 0.
#            For list of possible error codes refer to merchant documentation.
sub modifyDispositionValue{
    my($username, $password, $mtid, $subId, $amount, $currency) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateCurrency($currency, 'currency');

    return $client->modifyDispositionValue(
                            SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
                            SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                            SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
                            SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
                            SOAP::Data->name('{urn:pscservice}amount' => $amount)->type('float'),
                            SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'))
                            ->result;
}

# Calls the GetSerialNumbers web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param mtid
#             Merchant transaction id to use for the web service call. It must
#             be unique for each invocation.
#    @param subId
#             SubId to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @return Response of the GetSerialNumbers web service call. The call was
#            successful if resultCode and errorCode of the response are 0. For
#            list of possible error codes refer to merchant documentation.
sub getSerialNumbers{
    my($username, $password, $mtid, $subId, $currency) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateCurrency($currency, 'currency');

    return $client->getSerialNumbers(
                      SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
                      SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                      SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
                      SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
                      SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'))
                      ->result;
}

#  Calls GetDispositionRawState web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param mtid
#             Merchant transaction id to use for the web service call. It must
#             be unique for each invocation.
#    @param subId
#             SubId to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @return Response of the GetDispositionRawState web service call. The call
#            was successful if resultCode and errorCode of the response are 0.
#            For list of possible error codes refer to merchant documentation.
sub getDispositionRawState{
    my($username, $password, $mtid, $subId, $currency) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateCurrency($currency, 'currency');

    return $client->getDispositionRawState(
                          SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
                          SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                          SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
                          SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
                          SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'))
                          ->result;
}

# Calls ExecuteDebit web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param mtid
#             Merchant transaction id to use for the web service call. It must
#             be unique for each invocation.
#    @param subId
#             SubId to use for the web service call. The parameter is optional
#             and can be <code>null</code>.
#    @param amount
#             Amount to use for the web service call.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @param close
#             Close flag indicating if further debits will be executed or not.
#             If the close flag is 1 the disposition will be set to totally
#             consumed and no further debits are possible.
#    @param partialDebitId
#             PartialDebitId to use for the web service call. The parameter is
#             optional and can be <code>null</code>.
#    @ return Response of the ExecuteDebit web service call. The call was
#             successful if resultCode and errorCode of the response are 0. For
#             list of possible error codes refer to merchant documentation.
sub executeDebit{
    my($username, $password, $mtid, $subId, $amount, $currency, $close, $partialDebitId) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateStringNotNull($mtid, 'mtid');
    validateCurrency($currency, 'currency');

    return $client->executeDebit(
                         SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'),
                         SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                         SOAP::Data->name('{urn:pscservice}mtid' => $mtid)->type('string'),
                         SOAP::Data->name('{urn:pscservice}subId' => $subId)->type('string'),
                         SOAP::Data->name('{urn:pscservice}amount' => $amount)->type('float'),
                         SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'),
                         SOAP::Data->name('{urn:pscservice}close' => $close)->type('string'),
                         SOAP::Data->name('{urn:pscservice}partialDebitId' => $partialDebitId)->type('string'))
                         ->result;
}

#    Calls the GetMid SOPG web service method.
#    
#    @param username
#             Username to use for the web service call.
#    @param password
#             Password to use for the web service call.
#    @param currency
#             Currency to use for the web service call. It must contain a 3
#             letter ISO 4217 currency code.
#    @return Response containing mid configured for the given currency. The call
#            was successful if resultCode and errorCode of the response are 0.
#            For list of possible error codes refer to merchant documentation.
sub getMid {
    my($username, $password, $currency) = @_;
    validateStringNotNull($username, 'username');
    validateStringNotNull($password, 'password');
    validateCurrency($currency, 'currency');

    return $client->getMid(
                       SOAP::Data->name('{urn:pscservice}username' => $username)->type('string'), 
                       SOAP::Data->name('{urn:pscservice}password' => $password)->type('string'),
                       SOAP::Data->name('{urn:pscservice}currency' => uc $currency)->type('string'))
                       ->result;
}



#####################################################################################################################




# Creates a SOAP client connected to the given endpoint.
sub setup{
	$client = SOAP::Lite->proxy($_[0])->uri('pscservice');
	$mtid = time();
}
